"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const lodashEs = require("lodash-es");
const chokidar = require("chokidar");
const glob = require("glob");
const fs = require("fs");
const path = require("path");
const PLUGIN_NAME = "vite-i18n-gen-resources-type";
const BANNER = `/*! This file was generated by ${PLUGIN_NAME} check https://github.com/CRMChatReactComponent/vite-i18n-gen-resources-type. Please do not modify. */
`;
const rebuild = lodashEs.debounce(_rebuild, 1e3);
function GenI18nTypes(props) {
  const {
    watchFolder,
    outputFolder = path.dirname(watchFolder),
    isGenEnumFile = true,
    typesFileName = "genI18n.resources.d.ts",
    enumFileName = "genI18nEnum.ts",
    defaultLang = ""
  } = props;
  const enumFilePath = path.resolve(outputFolder, enumFileName);
  const typesFilePath = path.resolve(outputFolder, typesFileName);
  return [
    {
      name: "vite-gen-i18n-types",
      apply: "serve",
      configureServer({ watcher, config: { logger } }) {
        watcher.unwatch([enumFilePath, typesFilePath]);
        chokidar.watch(`${props.watchFolder}/**/*.json`, {
          depth: 2
        }).on("all", build);
        build();
        function build() {
          rebuild(
            props.watchFolder,
            typesFilePath,
            defaultLang,
            isGenEnumFile,
            enumFilePath
          );
        }
      }
    },
    {
      name: "vite-gen-i18n-types-build",
      apply: "build",
      options() {
        _rebuild(
          props.watchFolder,
          typesFilePath,
          defaultLang,
          isGenEnumFile,
          enumFilePath
        );
      }
    }
  ];
}
function _rebuild(watchFolder, typesFilePath, defaultLang, isGenEnumFile, enumFilePath) {
  const jsonGlob = `${watchFolder}/**/*.json`;
  const jsonFiles = glob.globSync(jsonGlob);
  const langs = {};
  for (const filePath of jsonFiles) {
    const lang = filePath.match(/\/locales\/([\w-]+)\//);
    const filename = filePath.match(/\/([\w-_]+)\.json$/);
    if (filename && lang) {
      langs[lang[1]] = langs[lang[1]] || {};
      try {
        langs[lang[1]][filename[1]] = JSON.parse(
          fs.readFileSync(filePath, "utf-8")
        );
      } catch (e) {
        console.error(
          `${PLUGIN_NAME}: the file cannot be parse to json, path:${filePath}`
        );
      }
    } else {
      console.error(
        `${PLUGIN_NAME}: the file cannot be parse to json, path:${filePath}`
      );
    }
  }
  if (Object.keys(langs).length > 0) {
    fs.writeFileSync(
      typesFilePath,
      genTypesFileContent(langs, defaultLang),
      "utf-8"
    );
    isGenEnumFile && fs.writeFileSync(enumFilePath, genEnumFileContent(langs), "utf-8");
    console.info(`${PLUGIN_NAME}: gen successfully`);
  } else {
    console.error(
      `${PLUGIN_NAME}: Unable to read **/*.json files under ${jsonGlob}, please check if the path is correct.`
    );
  }
}
function genEnumFileContent(langs) {
  let code = `${BANNER}export enum GenI18nEnum {`;
  for (const key of Object.keys(langs)) {
    const langKey = key.toUpperCase().replaceAll("-", "_");
    code += `
  ${langKey} = "${key}",`;
  }
  return code + "\n}\n";
}
function genTypesFileContent(langs, defaultLang) {
  defaultLang = defaultLang ? defaultLang : Object.keys(langs)[0];
  let code = `${BANNER}export interface I18nResourceInterface {`;
  for (const key of Object.keys(langs[defaultLang])) {
    const translationObj = langs[defaultLang][key];
    code += `
  ${key}: {`;
    for (const _key of Object.keys(translationObj)) {
      code += `
    ${_key}: "${translationObj[_key].replaceAll(`"`, "'")}";`;
    }
    code += "\n  };";
  }
  return code + "\n}\n";
}
exports.GenI18nTypes = GenI18nTypes;
